@@ -258,6 +258,9 @@ static struct genl_family ZZZ_genl_family __read_mostly = {
 	.hdrsize = NLA_ALIGN(GENL_MAGIC_FAMILY_HDRSZ),
 #endif
 	.maxattr = ARRAY_SIZE(drbd_tla_nl_policy)-1,
+#ifdef COMPAT_HAVE_GENL_FAMILY_PARALLEL_OPS
+	.parallel_ops = true,
+#endif
 };
 
 /*
